import json

from base64 import b64decode, b64encode
from onegov.gis.forms.widgets import CoordinatesWidget
from onegov.gis.models import Coordinates
from wtforms.fields import StringField


class CoordinatesField(StringField):
    """ Represents a single pair of coordinates with optional zoom and
    marker icon/color selection.

    In the browser and during transit the point is stored as a base64 encoded
    json string on a simple input field. For example::

        eydsYXQnOiA4LjMwNTc2ODY5MTczODc5LCAnbG.. (and so on)

        =>

        {'lat': 8.30576869173879, 'lon': 47.05183585, 'zoom': 10}

    For verification: This points to the Seantis office in Lucerne.

    For convenience, the coordinates are accessible with the
    :class:`onegov.gis.models.coordinates.Coordinates` class when 'data' is
    used.

    Note that this field doesn't work with the ``InputRequired`` validator.
    Instead the ``DataRequired`` validator has to be chosen.

    """

    widget = CoordinatesWidget()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.data = getattr(self, 'data', Coordinates())

    def _value(self):
        text = json.dumps(self.data.as_dict())
        text = b64encode(text.encode('ascii'))
        text = text.decode('ascii')

        return text

    def process_data(self, value):
        self.data = value and Coordinates(**value) or Coordinates()

    def populate_obj(self, obj, name):
        setattr(obj, name, self.data and self.data.as_dict() or None)

    def process_formdata(self, valuelist):
        if valuelist and valuelist[0]:
            text = b64decode(valuelist[0])
            text = text.decode('ascii')
            self.data = Coordinates(**json.loads(text))
        else:
            self.data = Coordinates()



import morepath


class MapboxApp(morepath.App):
    """ Provides mapbox integration
    :class:`onegov.core.framework.Framework` based applications.

    Doesn't do much except serve the mapbox public token, so we can store it
    in configuration and not with the source. Not that this token is inherently
    unsafe and must be the *public* token.

    Do not use private tokens!

    If we wanted to avoid this we would have to use a mapbox proxy server,
    which seems a bit too much. If we detect abuse of the public token we
    just switch to a new one. If it must be we can even automatically rotate
    the token regularly.

    """

    def configure_mapbox(self, **cfg):
        """ Configures the mapbox.

        The following configuration options are accepted:

        :mapbox_token:
            The public mapbox token to be used for the mapbox api.

        """
        assert cfg.get('mapbox_token', 'pk').startswith('pk'), """
            Only public mapbox tokens are allowed!
        """
        self.mapbox_token = cfg.get('mapbox_token', None)


@MapboxApp.tween_factory()
def inject_mapbox_api_token_tween_factory(app, handler):

    replacement = '<body data-mapbox-token="{}"'.format(app.mapbox_token)
    replacement = replacement.encode('utf-8')

    def inject_mapbox_api_token_tween(request):

        response = handler(request)

        if request.app.mapbox_token:
            response.body = response.body.replace(
                b'<body',
                replacement,
                1  # only replace the first occurrence
            )

        return response

    return inject_mapbox_api_token_tween



import json

from base64 import b64encode
from onegov.form import Form
from onegov.gis.models import Coordinates
from onegov.gis.forms import CoordinatesField


def test_coordinates_field():
    field = CoordinatesField().bind(Form(), 'coordinates')

    template = (
        '<input class="coordinates" id="coordinates" name="coordinates" '
        'type="text" value="{}">'
    )

    assert not field.data

    assert field.data.lat is None
    assert field.data.lon is None
    assert field() == template.format(b64encode(
        json.dumps({
            'lat': None,
            'lon': None,
            'zoom': None
        }).encode('ascii')
    ).decode('ascii'))

    field.process_formdata([b64encode(
        json.dumps({
            'lat': 47.05183585,
            'lon': 8.30576869173879
        }).encode('ascii')
    ).decode('ascii')])

    assert field.data.lat == 47.05183585
    assert field.data.lon == 8.30576869173879

    assert field() == template.format(b64encode(
        json.dumps({
            'lat': 47.05183585,
            'lon': 8.30576869173879,
            'zoom': None
        }).encode('ascii')
    ).decode('ascii'))

    field.process_data({
        'lat': 47.05183585,
        'lon': 8.30576869173879
    })

    assert field.data.lat == 47.05183585
    assert field.data.lon == 8.30576869173879

    class Test(object):
        pass

    test = Test()
    field.populate_obj(test, 'coordinates')

    assert test.coordinates['lat'] == 47.05183585
    assert test.coordinates['lon'] == 8.30576869173879

    field.data = Coordinates()
    field.populate_obj(test, 'coordinates')
    assert test.coordinates is None
